from typing import TYPE_CHECKING, Dict, Tuple

from django.db import models

if TYPE_CHECKING:
    from websocket_notifications.models import NotificationGroup


class NotificationGroupQuerySet(models.QuerySet):
    def get_or_create_for_user(self, user) -> Tuple["NotificationGroup", bool]:
        """Abstraction for create a notification channel from a given user."""
        if self.filter(user=user).exists():
            return self.filter(user=user).first(), False
        instance = self.model(user=user)
        instance.save()
        return instance, True

    def send(self, payload: Dict) -> None:
        """Sends to a queryset of notification channels."""
        for instance in self.iterator():
            instance.send(payload)



from django.conf import settings

# Needed to build and publish
# ------------------------------------------------------------------------------
SECRET_KEY = "websocket_notifications"

# Specific project configuration
# ------------------------------------------------------------------------------
MESSAGE_TYPE = getattr(
    settings, "WEBSOCKET_NOTIFICATIONS_MESSAGE_TYPE", "notification_message"
)



from typing import Dict

from asgiref.sync import async_to_sync
from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _
from model_utils.models import TimeStampedModel

from websocket_notifications.helpers import generate_random_code
from websocket_notifications.managers import NotificationGroupQuerySet
from websocket_notifications.settings import MESSAGE_TYPE


class NotificationGroup(TimeStampedModel):
    """This model is used to save an unique code for each user to use to connect for
    listenign notifications.
    """

    user = models.OneToOneField(
        settings.AUTH_USER_MODEL,
        verbose_name=_("user"),
        related_name="notification_group",
        on_delete=models.CASCADE,
    )
    code = models.CharField(_("code"), max_length=256, unique=True, blank=True)

    objects = NotificationGroupQuerySet.as_manager()

    class Meta:
        verbose_name = _("notification group")
        verbose_name_plural = _("notification groups")
        ordering = ["-created"]

    def generate_random_code(self) -> str:
        """Generates a random code using the user ID and the current time."""
        return generate_random_code(self.user.pk)

    def send(self, payload: Dict) -> None:
        """Sends the payload to the user."""
        from channels.layers import get_channel_layer

        channel_layer = get_channel_layer()
        async_to_sync(channel_layer.group_send)(
            self.code, {"type": MESSAGE_TYPE, "payload": payload}
        )

    def clean(self) -> None:
        # Generates the code if it doesn't exists
        if not self.code:
            self.code = self.generate_random_code()

    def save(self, *args, **kwargs):
        self.clean()
        super().save(*args, **kwargs)



from rest_framework import serializers, status, viewsets
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response

from websocket_notifications.models import NotificationGroup


class NotificationGroupSerializer(serializers.ModelSerializer):
    class Meta:
        model = NotificationGroup
        fields = ["code"]


class NotificationGroupViewSet(viewsets.ModelViewSet):
    queryset = NotificationGroup.objects.all()
    serializer_class = NotificationGroupSerializer
    permission_classes = [IsAuthenticated]

    def create(self, request, *args, **kwargs):
        instance = self.queryset.model.objects.filter(user=self.request.user).first()
        if not instance:
            instance, _ = self.queryset.model.objects.get_or_create_for_user(
                user=self.request.user
            )
        serializer = self.get_serializer(instance, data=request.data)
        serializer.is_valid(raise_exception=True)
        headers = self.get_success_headers(serializer.data)
        return Response(
            serializer.data, status=status.HTTP_201_CREATED, headers=headers
        )



from django.contrib.auth.mixins import LoginRequiredMixin
from django.shortcuts import render
from django.views import View

from websocket_notifications.models import NotificationGroup


class NotificationsListenerView(LoginRequiredMixin, View):
    """Testing view for listen to notifications."""

    template_name = "websocket_notifications/listener.html"

    def get(self, request):
        """Creates the channel on the fly."""
        group, _ = NotificationGroup.objects.get_or_create_for_user(user=request.user)
        context = {"group": group}
        return render(request, self.template_name, context=context)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['websocket_notifications',
 'websocket_notifications.api',
 'websocket_notifications.migrations',
 'websocket_notifications.snitch']

package_data = \
{'': ['*'], 'websocket_notifications': ['templates/websocket_notifications/*']}

install_requires = \
['channels>=2.4.0,<3.0.0',
 'django-model-utils>=4.0.0,<5.0.0',
 'django-snitch>=2.1.0,<3.0.0',
 'django>=3.0.7,<4.0.0',
 'djangorestframework>=3.11.0,<4.0.0']

setup_kwargs = {
    'name': 'django-websocket-notifications',
    'version': '1.1.0',
    'description': 'A Django package to handle notifications using Django Channels and WebSockets.',
    'long_description': '==============================\nDjango Websocket Notifications\n==============================\n\nA Django application to deliver user notifications made with \n`django-snitch <https://github.com/marcosgabarda/django-snitch>`_ using WebSockets.\n\n.. image:: https://travis-ci.org/marcosgabarda/django-websocket-notifications.svg?branch=master\n    :target: https://travis-ci.org/marcosgabarda/django-snitch\n\n.. image:: https://img.shields.io/badge/code_style-black-000000.svg\n   :target: https://github.com/ambv/black\n\n\nQuick start\n-----------\n\nThis applications works using django-channels, so, you need to integrate this with \nyour project before to integrate django-websocket-notifications. So, to make the \nquick start as quick and simple as possible, we\'ve made the following assumptions:\n\n* You already have integrated django-channels\n* You are using a channel layer, like Redis\n* You have a `routing.py` file\n* Your project uses DRF to deliver a RESTful API\n\n**1** Install using pip:\n\n.. code-block:: bash\n\n    pip install django-websocket-notifications\n\n**2** Add "websocket_notifications" to your INSTALLED_APPS settings like this:\n\n.. code-block:: python\n\n    INSTALLED_APPS += (\'websocket_notifications\',)\n\n**3** Add the routing patterns to your `routing.py` file:\n\n.. code-block:: python\n\n    from channels.auth import AuthMiddlewareStack\n    from channels.routing import ProtocolTypeRouter, URLRouter\n    from websocket_notifications.routing import websocket_urlpatterns\n\n\n    application = ProtocolTypeRouter(\n        {"websocket": AuthMiddlewareStack(URLRouter(websocket_urlpatterns)),}\n    )\n\n**4** (Optional) In order to test the integration, you can add the following view to your `urls.py` file to be able to access to a testing view:\n\n.. code-block:: python\n\n    urlpatterns += [\n        path(\n            "websocket-notifications/",\n            include(\n                "websocket_notifications.urls",\n                namespace="websocket_notifications",\n            ),\n        ),\n    ]\n\nNow, you can access to `/websocket-notifications/listener/` to check the integration.\n\n**5** Add the ViewSet to the DRF router:\n\n.. code-block:: python\n\n    from websocket_notifications.api.rest_framework import NotificationGroupViewSet\n\n\n    router = routers.DefaultRouter()\n    router.register("websocket-notifications/groups", viewset=NotificationGroupViewSet)\n\n**6** Integrate with `django-snitch`:\n\n.. code-block:: python\n\n    from websocket_notifications.snitch.backends import WebSocketNotificationBackend\n\n\n    @snitch.register(EVENT)\n    class MyEventHandler(snitch.EventHandler):\n        ephemeral = True\n        notification_backends = [WebSocketNotificationBackend]',
    'author': 'Marcos Gabarda',
    'author_email': 'hey@marcosgabarda.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/marcosgabarda/django-websocket-notifications',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from django.urls import path

from websocket_notifications.views import NotificationsListenerView

app_name = "websocket_notifications"
urlpatterns = [path("listener/", NotificationsListenerView.as_view(), name="listener")]
